"""distutils

The main package for the Python Module Distribtion Utilities.  Normally
used from a setup script as

   from distutils.core import setup

   setup (...)
"""

__revision__ = "$Id: __init__.py,v 1.8 2000/06/30 02:54:36 gward Exp $"

__version__ = "0.9"
