#
# Local customizations
#

# Where to find the Universal Header include files:
MWERKSDIR="flap:Metrowerks:Metrowerks CodeWarrior:"
INCLUDEDIR=MWERKSDIR + "MacOS Support:Headers:Universal Headers:"

# Where to put the python definitions file:
TOOLBOXDIR="flap:Jack:Python:Mac:Lib:lib-toolbox:"

# Creator for C files:
CREATOR="CWIE"
